package net.hoosiertransfer.EaglerXMicrosoft.steps;

import java.util.function.Consumer;

import org.json.JSONObject;

import net.hoosiertransfer.EaglerXMicrosoft.ApplicationDetails;
import net.hoosiertransfer.EaglerXMicrosoft.OAuthEnvironment;
import net.hoosiertransfer.EaglerXMicrosoft.network.PostRequest;

public class StepMsaDeviceCode extends AbstractStep<StepMsaDeviceCode.MsaDeviceCode> {
    public StepMsaDeviceCode(ApplicationDetails applicationDetails) {
        super(applicationDetails);
    }

    public MsaDeviceCode execute(Consumer<MsaDeviceCode> consumer) {
        PostRequest postRequest = new PostRequest();
        postRequest.put("client_id", this.applicationDetails.CLIENT_ID);
        postRequest.put("scope", this.applicationDetails.SCOPE);
        if (this.applicationDetails.OAUTH_ENVIRONMENT == OAuthEnvironment.LIVE) {
            postRequest.put("response_type", "device_code");
        }

        JSONObject response = postRequest.send(this.applicationDetails.OAUTH_ENVIRONMENT.getDeviceCodeUrl());

        MsaDeviceCode msaDeviceCode = new MsaDeviceCode(
                System.currentTimeMillis() + response.getLong("expires_in") * 1000,
                response.getLong("interval") * 1000, response.getString("device_code"), response.getString("user_code"),
                response.getString("verification_uri"));

        consumer.accept(msaDeviceCode);
        return msaDeviceCode;
    }

    public static class MsaDeviceCode {
        long expireTimeMs;
        long intervalMs;
        String deviceCode;
        String userCode;
        String verificationUri;

        public MsaDeviceCode(long expireTimeMs, long intervalMs, String deviceCode, String userCode,
                String verificationUri) {
            this.expireTimeMs = expireTimeMs;
            this.intervalMs = intervalMs;
            this.deviceCode = deviceCode;
            this.userCode = userCode;
            this.verificationUri = verificationUri;
        }

        public String getUserCode() {
            return this.userCode;
        }

        public String getVerificationUri() {
            return this.verificationUri;
        }

    }
}
